#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Main module file for the Movie() and Title() objects'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces
from media.data.dates import ExactDate, RangeDate


class Instance():
    '''
    Instance tracks data on a single physical copy of
    media.  It tracks a local filing id value, an
    acquisition, and a value tracking the condition
    of the instance.
    '''
    def __init__(self, in_element):
        self.local_id = None
        self.acquisition = None
        self.condition = None
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'localId':
                self.local_id = child.text
            elif child.tag == Namespaces.nsf('media') + 'acquisition':
                self.acquisition = Aquisition(child)
            elif child.tag == Namespaces.nsf('media') + 'condition':
                self.condition = Condition(child)


class Aquisition():
    '''
    Acquisition details the process of how an instance was acquired.
    All aquisitions have a date, and further data that is either
    a purchase or a gift.
    '''
    def __init__(self, in_element):
        self.date = None
        self.acq = None
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'date':
                date_element = child[0]
                if date_element.tag == Namespaces.nsf('media') + 'exact':
                    self.date = ExactDate(date_element)
                elif date_element.tag == Namespaces.nsf('media') + 'from':
                    self.date = RangeDate(date_element, child[1])
            if child.tag == Namespaces.nsf('media') + 'purchase':
                self.acq = Purchase(child)
            elif child.tag == Namespaces.nsf('media') + 'gift':
                self.acq = Gift(child)

    def __str__(self):
        return f"{self.acq!s}"


class Gift():
    '''
    An instance that was acquired as a gift.
    '''
    def __init__(self, in_element):
        self.g_from = ''
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'from':
                self.g_from = child.text

    def __str__(self):
        return f"Gift (from {self.g_from})"


class Purchase():
    '''
    A pysical media instance that was purchased.
    '''
    def __init__(self, in_element):
        self.retailer = ''
        self.location = ''
        self.price = 0.0
        self.quality = ''
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'retailer':
                self.retailer = child.text
            if child.tag == Namespaces.nsf('media') + 'location':
                self.location = child.text
            if child.tag == Namespaces.nsf('media') + 'price':
                self.price = float(child.text)
            if child.tag == Namespaces.nsf('media') + 'quality':
                self.quality = Namespaces.ns_strip(child[0].tag)

    def __str__(self):
        out = 'Purchase'
        if self.price > 0:
            out += f" {self.price:.2f}"
        r_string = self.retailer.strip()
        if r_string:
            out += f" ({r_string})"
        return out


class Condition():
    '''
    Identifies the condition of the physical media.
    '''
    def __init__(self, in_element):
        self.status = ''
        self.notes = ''
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'status':
                self.status = Namespaces.ns_strip(child[0].tag)
            if child.tag == Namespaces.nsf('media') + 'notes':
                self.notes = child.text



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Main module file for the Movie() and Title() objects'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces
from media.data.media.contents.generic.catalog import (
        Title, TitleValueException, Catalog
        )
from media.data.media.contents.genericv.story import Story
from media.data.media.contents.genericv.crew import Crew
from media.data.media.contents.genericv.technical import Technical
from media.data.media.contents.movie.classification import Classification


class Movie():
    '''Movie object'''
    def __init__(self, in_chunk):
        self.title = None
        self.catalog = None
        self.technical = None
        self.crew = None
        self.unique_key = ""
        self._process(in_chunk)

    def _process(self, in_chunk):
        for child in in_chunk:
            if child.tag == Namespaces.nsf('movie') + 'title':
                try:
                    self.title = Title(child.text)
                except TitleValueException as tve:
                    raise MovieException(tve.message) from tve
            if child.tag == Namespaces.nsf('movie') + 'catalog':
                self.catalog = Catalog(child)
            if child.tag == Namespaces.nsf('movie') + 'classification':
                self.classification = Classification(child)
            if child.tag == Namespaces.nsf('movie') + 'technical':
                self.technical = Technical(child)
            if child.tag == Namespaces.nsf('movie') + 'story':
                self.story = Story(child)
            if child.tag == Namespaces.nsf('movie') + 'description':
                self.story = Story(child)
            if child.tag == Namespaces.nsf('movie') + 'crew':
                self.crew = Crew(child)
        if self.title is not None:
            self._build_unique_key()

    def _build_unique_key(self):
        ukv = None
        cpy = None
        if self.catalog is not None:
            if self.catalog.alt_titles is not None:
                if self.catalog.alt_titles.variant_sort is True:
                    self.unique_key = \
                            self.catalog.alt_titles.variant_title.sort_title
                else:
                    self.unique_key = self.title.sort_title
            if self.catalog.copyright is not None:
                cpy = str(self.catalog.copyright.year)
            else:
                cpy = "0000"
            if self.catalog.unique_index is not None:
                ukv = str(self.catalog.unique_index.index)
            else:
                ukv = "1"
            self.unique_key += "-" + cpy + "-" + ukv
        else:
            self.unique_key = self.title.sort_title + "-0000-1"

    def __hash__(self):
        return hash(self.unique_key)

    def __lt__(self, other):
        return self.unique_key < other.unique_key

    def __gt__(self, other):
        return self.unique_key > other.unique_key

    def __eq__(self, other):
        return self.unique_key == other.unique_key


class MovieException(Exception):
    '''Exception raised when there is an issue with a movie.'''
    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return self.message



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Simple command line client to list out movies'''

# pylint: disable=R0801

# brief movie report
import os
import argparse
import media.fmt.text.movie
from media.tools.common import (
        load_media_dev, compile_movies, random_sample_list
        )


def show_movies(in_list, in_args):
    '''
    Show every movie entry, ordered by either title or runtime.
    '''
    sort_field = in_args.sort
    p_breaks = in_args.pagebreaks
    if sort_field == 'runtime':
        order_list = sorted(in_list, key=lambda x: x.runtime)
    else:
        order_list = sorted(in_list, key=lambda x: x.title_key)
    for movie_out in order_list:
        print(movie_out)
        if p_breaks:
            print(chr(12))


def prep_list(in_movies):
    '''
    Build the list of output entries.
    '''
    out_l = []
    for movie in in_movies:
        out_l.append(media.fmt.text.movie.Brief(movie))
    return out_l


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    parser.add_argument('--random', type=int, help='print X random entries')
    parser.add_argument('--sort', choices=['title', 'runtime'],
                        help='Sort field')
    parser.add_argument('--pagebreaks', action=argparse.BooleanOptionalAction,
                        help='Page break after every movie.')
    args = parser.parse_args()
    moviepath = args.mediapath or os.environ['MEDIAPATH']
    if not moviepath:
        parser.print_help()
    devices = load_media_dev(moviepath)
    all_movies = compile_movies(devices)
    if args.random:
        chunks = prep_list(random_sample_list(all_movies, args.random))
    else:
        chunks = prep_list(all_movies)
    show_movies(chunks, args)



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Module for movie classification
'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces


class Classification():
    '''
    The classification object records genre information that presents
    a high level overview of the film.
    '''
    def __init__(self, in_chunk):
        self.category = None
        self.genres = None
        if in_chunk is not None:
            self._process(in_chunk)

    def _process(self, in_chunk):
        for child in in_chunk:
            ele_name = Namespaces.ns_strip(child.tag)
            if ele_name == 'category':
                self.category = Category(child)
            elif ele_name == 'genres':
                self.genres = Genres(child)


class Category():
    '''
    A simple breakdown as to whether a film is fiction or non-fiction
    '''
    FICTION = 1
    NONFICTION = 0
    UNKNOWN = -1

    def __init__(self, in_chunk):
        self.value = Category.UNKNOWN
        if in_chunk is not None:
            child = in_chunk[0]
            ele_name = Namespaces.ns_strip(child.tag)
            if ele_name == 'fiction':
                self.value = Category.FICTION
            elif ele_name == 'nonfiction':
                self.value = Category.NONFICTION

    def __str__(self):
        '''Return a string representation of the value'''
        s_value = ""
        if self.value == Category.FICTION:
            s_value = "FICTION"
        elif self.value == Category.NONFICTION:
            s_value = "NONFICTION"
        return s_value


class Genres():
    '''
    Genres and subgenres information on a film.
    '''
    def __init__(self, in_chunk):
        self.primary = None
        self.secondary = []
        self.setting = None
        self.specific = None
        self.subgenres = []
        if in_chunk is not None:
            self._process(in_chunk)

    def _process(self, in_chunk):
        for child in in_chunk:
            ele_name = Namespaces.ns_strip(child.tag)
            if ele_name == 'primary':
                self.primary = child.text
            elif ele_name == 'secondary':
                self.secondary.append(child.text)
            elif ele_name == 'setting':
                self.setting = child.text
            elif ele_name == 'specific':
                self.specific = child.text
            elif ele_name == 'subgenres':
                self.subgenres = Genres._process_sg(child)

    @classmethod
    def _process_sg(cls, in_chunk):
        subgenres = []
        for child in in_chunk:
            ele_name = Namespaces.ns_strip(child.tag)
            if ele_name == 'subgenre':
                subgenres.append(child.text)
        return subgenres



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Code to handle technical aspects of a vidsual format media
(movies, television)
'''

# pylint: disable=too-few-public-methods

import re
from datetime import timedelta
from media.xml.namespaces import Namespaces


class Technical():
    '''
    Technical element block.

    Covering visual format, palette, and runtime.
    '''
    def __init__(self, in_element):
        self.runtime = None
        self.palette = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'runtime':
                self.runtime = Runtime(child)


class Runtime():
    '''
    Class for handling the runtime interval of a movie or TV show.
    '''
    def __init__(self, in_element):
        self.overall = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'overall':
                dur_string = child.text
                dur_value = process_iso_duration(dur_string)
                self.overall = dur_value


def process_iso_duration(in_string):
    '''
    Process an ISO duration value

    Ex: like P01H20M37S (1 hour, 20 minutes, 37 seconds)
    '''
    duration_regex = r"PT(?P<hours>\d{1,2}H)?(?P<minutes>\d{1,2}M)?" +\
                     r"(?P<seconds>\d{1,2}S)?"
    dur_match = re.match(duration_regex, in_string)
    if dur_match is not None:
        if dur_match.group('hours') is not None:
            in_hours = int(dur_match.group('hours').rstrip('H'))
        else:
            in_hours = 0
        if dur_match.group('minutes') is not None:
            in_minutes = int(dur_match.group('minutes').rstrip('M'))
        else:
            in_minutes = 0
        if dur_match.group('seconds') is not None:
            in_seconds = int(dur_match.group('seconds').rstrip('S'))
        else:
            in_seconds = 0
        return timedelta(hours=in_hours,
                         minutes=in_minutes,
                         seconds=in_seconds)
    return None



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Module to report on all keywords within a movie
'''

# pylint: disable=R0801

import os
import argparse
from media.tools.common import (
        load_media_dev, compile_movies, random_sample_list
        )


class KeywordTitleMap():
    '''
    One keyword object with a list of titles that utilize that keyword.


    '''
    def __init__(self, in_keyword, in_title):
        self.keyword = in_keyword
        self.titles = [in_title]

    def add_title(self, in_title):
        '''Add a movie title to an existing keyword object.'''
        self.titles.append(in_title)

    @classmethod
    def header(cls):
        '''Print a simple header.'''
        out = f"{'Keyword':45s} {'Title'}\n" + \
              f"{'=' * 45} {'=' * 25}\n"
        return out

    def __str__(self):
        '''Print out the keyword with a sorted title list'''
        out = ''
        line = 1
        for title_i in sorted(self.titles):
            if line == 1:
                out += f"{self.keyword.detail():45s} {title_i!s}\n"
            else:
                out += f"{' ':45s} {title_i!s}\n"
            line += 1
        return out

    def __lt__(self, other):
        return self.keyword.sort_value < other.keyword.sort_value

    def __gt__(self, other):
        return self.keyword.sort_value > other.keyword.sort_value

    def __eq__(self, other):
        return self.keyword.sort_value == self.keyword.sort_value


def grab_keywords(movies):
    '''
    Build a dictionary of all keywords.

    For every movie, pull all of the keywords.

    For existing keywords in the dictionary, add the title to
    the existing object.
    '''
    kw_dict = {}
    for movie in movies:
        kw_pull = None
        story = None
        if movie.story is not None:
            story = movie.story
        elif movie.description is not None:
            story = movie.description
        if story.keywords is not None:
            kw_pull = story.keywords.all()
            for kw_i in kw_pull:
                kw_detail = kw_i.detail()
                if kw_detail in kw_dict:
                    kw_dict[kw_detail].add_title(movie.title)
                else:
                    kw_dict[kw_detail] = \
                            KeywordTitleMap(kw_i, movie.title)
    return list(kw_dict.values())


def list_keywords(keywords):
    '''Print the keywords that are passed along.'''
    print(KeywordTitleMap.header(), end='')
    for kw_i in sorted(keywords):
        print(kw_i, end='')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    parser.add_argument('--random', type=int, help='show X keywords')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    devices = load_media_dev(mediapath)
    all_movies = compile_movies(devices)
    all_keywords = grab_keywords(all_movies)
    if args.random:
        rand_limit = args.random
        list_keywords(random_sample_list(all_keywords, rand_limit))
    else:
        list_keywords(all_keywords)



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
List out a breakdown of movie time lengths into buckets.
'''

# pylint: disable=R0801

from datetime import timedelta

import os
import argparse
from media.tools.common import (
        load_media_dev, compile_movies, random_sample_list
        )
from media.tools.movies.list import prep_list
from media.tools.movies.genrebreakdown import proportion_bar


class BucketManager():
    '''
    Holds all bucket objects, and performs
    reporting, and data aggregation operations across all of them.
    '''
    def __init__(self, in_limit=10):
        self.limit = in_limit
        self.buckets = []
        self.average = None
        self.median = None
        self.median_title = ''
        self.movie_count = 0
        self.interval = 0

    def initialize_bucket_ranges(self, in_list):
        '''
        Set up ever bucket object (up to limit), and
        configure the range values of each object.
        '''
        shortest = in_list[0].runtime
        longest = in_list[-1].runtime
        interval = self.get_bucket_interval(shortest, longest)
        # print(f"Interval is {interval}\n")
        start_i = shortest
        for _ in range(1, self.limit + 1):
            self.buckets.append(Bucket(start_i, interval, len(in_list)))
            start_i += timedelta(seconds=interval)

    def get_bucket_interval(self, shortest, longest):
        '''
        Determine the bucket interval length by subtracting the shortest
        movie runtime from the longest movie runtime, then divide it
        by the number of requested buckets.  This requires turning the
        runtimes into integers that represent all of the seconds in the
        timedelta object.

        Input is two timedelta objects, and the return value is another
        timedelta object.
        '''
        delta_range = longest - shortest
        delta_int = int(delta_range.total_seconds())
        range_int = int(delta_int / self.limit) + 1
        self.interval = timedelta(seconds=range_int)
        return range_int

    def install_in_buckets(self, in_list):
        '''
        For every passed movie, install it into a bucket by
        passing it to every bucket object, knowing that
        it only gets accepted into a single bucket.

        There are probably ways to make this code more
        efficient.
        '''
        total_r = 0
        for movie_i in in_list:
            for bucket_i in self.buckets:
                if bucket_i.is_in_range(movie_i.runtime):
                    bucket_i.movies.append(movie_i)
                    break
            total_r += movie_i.runtime.total_seconds()
        self.movie_count = len(in_list)
        self.calculate_average(total_r)
        self.calculate_median(in_list)

    def calculate_average(self, total_runtimes):
        '''
        Caculate the average time of all the movies.
        '''
        average_seconds = int(total_runtimes / self.movie_count)
        self.average = timedelta(seconds=average_seconds)

    def calculate_median(self, in_list):
        '''
        Calculate the median time of all the movies, and
        report on which titles(s) fit the median.
        '''
        if self.movie_count % 2 == 0:
            median1 = int(self.movie_count / 2)
            median2 = median1 + 1
            total_median_seconds = in_list[median1].runtime.total_seconds() + \
                in_list[median2].runtime.total_seconds()
            self.median = timedelta(seconds=int(total_median_seconds / 2))
            self.median_title = f"{in_list[median1].movie.title!s:s} / " + \
                f"{in_list[median2].movie.title!s:s}"
        else:
            median1 = int(self.movie_count / 2)
            self.median = in_list[median1].runtime
            self.median_title = f"{in_list[median1].movie.title!s:s}"

    def report_buckets(self):
        '''
        Generate a report on all buckets, by sending the report command
        to every bucket in the list.  Then provide summary data
        at the bottom of the report.
        '''
        print(Bucket.header())
        for bucket_i in self.buckets:
            print(bucket_i.report())
        print(Bucket.hdr_lines())
        print(f"         Movie count : {self.movie_count}")
        print(f"Average film runtime : {self.average!s:s}")
        print(f" Median film runtime : {self.median!s:8s} " +
              f"({self.median_title})")
        print(f"     Interval Length : {self.interval!s:s}")
        print(f"        Bucket Count : {len(self.buckets)}")


class Bucket():
    '''
    A single bucket designed to hold movie objects.

    It has a low end limit and a high end limit, and then uses those
    timedelta values to see if a movie should go in that bucket
    based on the length of the movie.
    '''
    def __init__(self, in_low_limit, in_range,  in_total_count):
        self.low_limit = in_low_limit
        self.high_limit = in_low_limit + timedelta(seconds=in_range)
        self.total_count = in_total_count
        self.movies = []

    def is_in_range(self, in_timedelta):
        '''
        Simple check to see if a passed movie falls within the bucket
        range.
        '''
        return bool(self.low_limit <= in_timedelta < self.high_limit)

    def range_print(self):
        '''
        A debugging method to examine the bucket status.
        '''
        return f"Low   : {self.low_limit}\n" + \
            f"High  : {self.high_limit}\n" + \
            f"Count : {len(self.movies)} / {self.total_count}\n\n"

    @classmethod
    def header(cls):
        '''
        Sends out a report header.
        '''
        out = f"{'Random Title':50s} " + \
              f"{'From':8s} {'To':8s} " + \
              f"{'Count':5s} {'Perc':6s} {'Ratio':50s}\n" + \
              cls.hdr_lines()
        return out

    @classmethod
    def hdr_lines(cls):
        '''
        Sends out formatting lines used in the report header.
        '''
        return f"{'-' * 50} {'-'*8} {'-'*8} {'-'*5} {'-'*6} {'-'*50}"

    def report(self):
        '''
        Generate a report entry on a single bucket object.
        Show the range limits (low and high, the number of
        movies, the percentage based on a total, and a tick bar
        for illustrative purposes.  Also show a random movie title.
        '''
        tick_bar = proportion_bar(len(self.movies), self.total_count)
        movie = random_sample_list(self.movies, 1)
        if len(movie) > 0:
            mov_title = str(movie[0].movie.title)
            perc = float(len(self.movies) / self.total_count * 100)
        else:
            mov_title = " << None >>"
            perc = 0
        return f"{str(mov_title):50s} {self.low_limit!s:>8s} " + \
               f"{self.high_limit!s:>8s} {len(self.movies):5d} " + \
               f"{perc:5.1f}% {tick_bar}"


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    parser.add_argument('--buckets', type=int, help='number of sample buckets')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    devices = load_media_dev(mediapath)
    all_movies = compile_movies(devices)
    list_entries = prep_list(all_movies)
    sorted_movies = sorted(list_entries, key=lambda x: x.runtime)
    if args.buckets:
        buckets = BucketManager(args.buckets)
    else:
        buckets = BucketManager()
    buckets.initialize_bucket_ranges(sorted_movies)
    buckets.install_in_buckets(sorted_movies)
    buckets.report_buckets()



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''The medium release class'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces
import media.data.media.medium.inventory as MI


class ProductSpecs():
    '''Structure for all elements pertaining to product identification.'''
    def __init__(self, in_chunk):
        self.inventory = None
        self.dimensions = None
        self._process(in_chunk)

    def _process(self, in_chunk):
        for child in in_chunk:
            if child.tag == Namespaces.nsf('media') + 'inventory':
                self.inventory = Inventory(child)
            if child.tag == Namespaces.nsf('media') + 'dimensions':
                self.dimensions = Dimensions(child)


class Inventory():
    '''
    Inventory of all items in the package.
    '''
    def __init__(self, in_element):
        self.inventory = []
        self._process(in_element)

    def _process(self, in_element):
        if len(in_element) > 0:
            for child in in_element:
                new_t = Namespaces.ns_strip(child.tag)
                if new_t == 'box':
                    self.inventory.append(MI.Box(child))
                if new_t in MI.Containers:
                    sub_c = MI.Containers[new_t](child)
                    # sub_c.__init__(child)
                    self.inventory.append(sub_c)

    def __str__(self):
        out = ''
        for itm in self.inventory:
            out += f"{itm!s}"
        return out


class Dimensions():
    '''
    Overall dimensions of the physical media.
    '''
    def __init__(self, in_element):
        self.length = 0.0
        self.width = 0.0
        self.height = 0.0
        self.weight = 0.0
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'size':
                self._process_size(child)
            elif child.tag == Namespaces.nsf('media') + 'weight':
                self._process_weight(child)

    def _process_size(self, in_element):
        '''
        Explanation on legth, width, and height:

        length and width represent the measurments of
        the package if it was upright and lying flat on its back, which
        means that height refers to the height of the spine.

        In most measurements, length is used to refer to the
        higher value of measurement between length and width.

        Lying flat on its back, a typical US blu-ray case is 6.75
        inches in length (or 6.75 inches tall), and
        5.4 inches in width.  That leaves the final
        dimension as height, which is usually 0.5 inches.

        The dimensions only get set if both length and
        width are positive values.  The height value
        is an optional value because some storage
        units, like Envelopes have a height value
        so low, it's almost not worth recording.
        '''
        if 'length' in in_element.attrib:
            length = float(in_element.attrib['length'])
        if 'width' in in_element.attrib:
            width = float(in_element.attrib['width'])
        if 'height' in in_element.attrib:
            height = float(in_element.attrib['height'])
        if length > 0 and width > 0:
            self.length = length
            self.width = width
            if height > 0:
                self.height = height

    def _process_weight(self, in_element):
        if in_element.text:
            weight = float(in_element.text)
            if weight > 0:
                self.weight = weight



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''The medium release class'''

# pylint: disable=too-few-public-methods
# pylint: disable=consider-using-dict-items

import sys
from media.xml.namespaces import Namespaces


class AbstractPackage():
    '''Abstract class for packaging.'''
    def __init__(self):
        self.media = []
        self.type_s = 'UNDEF'

    def subcontainers_list(self):
        '''
        Return all containers in a
        package.
        '''
        out = []
        for m_item in self.media:
            if issubclass(m_item.__class__, AbstractCase):
                out.append(m_item)
        return out

    def media_string_count_hash(self):
        '''
        Iterates through all objects in
        the inventory.  If the object is a media
        type, it is placed in a hash table to
        record the type, and build an inventory
        count.
        '''
        m_table = {}
        for m_item in self.media:
            if issubclass(m_item.__class__, AbstractMedia):
                media_str = str(m_item)
                if media_str in m_table:
                    m_table[media_str] += 1
                else:
                    m_table[media_str] = 1
        return m_table

    def media_string_count_list(self):
        '''
        Return a formatted list of strings reporitng
        inventory breakdown.  IE, if a container has
        two DVDs and a blu-ray, it returns two strings,
        one for each item type, with a count integer
        if the count is greater than 1.
        '''
        m_list = []
        m_table = self.media_string_count_hash()
        for m_item in m_table:
            if m_table[m_item] > 1:
                m_list.append(f"{m_item} ({m_table[m_item]})")
            else:
                m_list.append(f"{m_item}")
        return m_list

    def __str__(self):
        return self.type_s


class AbstractCase(AbstractPackage):
    '''Abstract 2nd level case.'''


class Box(AbstractPackage):
    '''Product case that can hold other cases.'''
    def __init__(self, in_element):
        super().__init__()
        self.type_s = 'Box'
        self._process(in_element)

    def _process(self, in_element):
        '''
        The _process method for Box is unique
        because it's possible for a box
        to contain other containers.

        This method could conceivably be
        moved to an abstract class if
        there were other weird package
        options that could contain other cases.
        '''
        if len(in_element) > 0:
            for child in in_element:
                child_tag = Namespaces.ns_strip(child.tag)
                if child_tag in Containers:
                    sub_c = Containers[child_tag](child)
                    self.media.append(sub_c)


class Case(AbstractCase):
    '''Generic product case.'''
    def __init__(self, in_element):
        super().__init__()
        self.type_s = 'Case'
        self.slipcover = False
        self._process(in_element)

    def _process(self, in_element):
        '''
        Identify all media or other pieces inside
        the container and put them in the media
        array.
        '''
        if 'slipcover' in in_element.attrib:
            self.slipcover = True
        if len(in_element) > 0:
            for child in in_element:
                child_tag = Namespaces.ns_strip(child.tag)
                if child_tag in Media:
                    sub_m = Media[child_tag](child)
                    self.media.append(sub_m)


class Snapcase(AbstractCase):
    '''old style cardboard and plastic case'''
    def __init__(self, in_element):
        super().__init__()
        self.type_s = 'Snapcase'
        self._process(in_element)

    def _process(self, in_element):
        '''
        Identify all media or other pieces inside
        the container and put them in the media
        array.
        '''
        if len(in_element) > 0:
            for child in in_element:
                c_tag = Namespaces.ns_strip(child.tag)
                if c_tag in Media:
                    sub_m = Media[c_tag](child)
                    self.media.append(sub_m)


class DigiBook(AbstractCase):
    '''media case that typically has 1 disc and 1 book.'''
    def __init__(self, in_element):
        super().__init__()
        self.type_s = 'Digibook'
        self._process(in_element)

    def _process(self, in_element):
        '''
        Identify all media or other pieces inside
        the container and put them in the media
        array.
        '''
        if len(in_element) > 0:
            for child in in_element:
                c_tag = Namespaces.ns_strip(child.tag)
                if c_tag in Media:
                    sub_m = Media[c_tag](child)
                    self.media.append(sub_m)


class Envelope(AbstractCase):
    '''cardboard envelope.'''
    def __init__(self, in_element):
        super().__init__()
        self.type_s = 'Envelope'
        self._process(in_element)

    def _process(self, in_element):
        '''
        Identify all media or other pieces inside
        the container and put them in the media
        array.
        '''
        if len(in_element) > 0:
            for child in in_element:
                c_tag = Namespaces.ns_strip(child.tag)
                if c_tag in Media:
                    sub_m = Media[c_tag](child)
                    self.media.append(sub_m)


class AbstractMedia():
    '''Abstract media object.'''
    def __init__(self, in_element):
        self.media_id = 0
        self.type_s = 'UNDEF'
        if in_element:
            pass

    def __str__(self):
        return self.type_s


class VideoDiscMedia(AbstractMedia):
    '''Video disc class.'''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.radius = 5


class DVD(VideoDiscMedia):
    '''DVD class.'''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.type_s = 'DVD'


class BluRay(VideoDiscMedia):
    '''Blu-Ray disc class.'''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.type_s = 'Blu-Ray'


class UltraHD(VideoDiscMedia):
    '''UltraHD disc class.'''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.type_s = 'UltraHD'


class Booklet(AbstractMedia):
    '''Booklet  class.'''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.type_s = 'Booklet'


class CodeSheet(AbstractMedia):
    '''Code sheet for downloading media.'''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.type_s = 'Codesheet'


#
# The following dictionaries associate the element names
# with the appropriate classes they represent.
#


Containers = {
        'case': getattr(sys.modules[__name__], 'Case'),
        'snapcase': getattr(sys.modules[__name__], 'Snapcase'),
        'digibook': getattr(sys.modules[__name__], 'DigiBook'),
        'envelope': getattr(sys.modules[__name__], 'Envelope'),
        }

Media = {
        'ultrahd': getattr(sys.modules[__name__], 'UltraHD'),
        'bluray': getattr(sys.modules[__name__], 'BluRay'),
        'dvd': getattr(sys.modules[__name__], 'DVD'),
        'booklet': getattr(sys.modules[__name__], 'Booklet'),
        'codesheet': getattr(sys.modules[__name__], 'CodeSheet'),
        }



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''The medium release class'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces


class Release():
    '''
    The release is the basic description of a physical release
    of a work of art.

    For a movie that could be a film canister, or a DVD.
    For a story the release could be a book.
    For a piece of music, that release could be an audio CD.

    The release describes the type of release it is, who
    published it, and the date it was released.
    '''
    def __init__(self, in_element):
        self.type = None
        self.publisher = None
        self._process(in_element)

    def _process(self, in_element):
        '''
        The publisher tag is easy to handle.
        The type tag is tricky because it can only
        contain a single XML element.
        '''
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'type':
                if len(child) == 1:
                    self.type = Namespaces.ns_strip(child[0].tag)
            elif child.tag == Namespaces.nsf('media') + 'publisher':
                self.publisher = child.text
        if self.type is None:
            raise ReleaseException('No media release type set.')


class ReleaseException(Exception):
    '''
    The ReleaseException class is thrown when there is
    incomplete data in the Release data structure.
    '''
    def __init__(self, in_message):
        super().__init__(in_message)
        self.message = in_message

    def __str__(self):
        return self.message


class FormalType():
    '''Static output representation on media types.'''
    f_map = {
            "cd": "CD",
            "casette": "Casette",
            "record": "Record",
            "dvd": "DVD",
            "bluray": "Blu-Ray",
            "bluray3d": "Blu-Ray 3D",
            "hddvd": "HD-DVD",
            "ultrahd": "Ultra HD",
            "vhs": "VHS",
            "beta": "Betamax"
        }

    @classmethod
    def formal_convert(cls, in_type):
        '''
        Return the type value in a more friendly string.
        '''
        if in_type in cls.f_map:
            return FormalType.f_map[in_type]
        return "UNKNOWN"



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Common routines for command line tools'''

import random
import media.fileops.scanner
import media.fileops.loader
import media.fileops.repo
from media.fileops.filenames import FilenameMatches

# Walker module walks the filesystem
# Loader module reads in the discovered files


def load_media_dev(in_path):
    '''Identify suitable files and load them up'''
    repo = media.fileops.repo.Repo(in_path)
    repo.scan()
    loader = media.fileops.loader.Loader()
    m_devices = loader.load_media(repo, FilenameMatches.Movie_Media)
    return m_devices


def compile_movies(media_devices):
    '''Extract movies from devices'''
    movies = []
    for m_dev in media_devices:
        for movie in m_dev.contents:
            if movie not in movies:
                movies.append(movie)
    return movies


def random_sample_list(in_list, rand_limit):
    '''
    Take a large list, and turn it into a random sample.
    '''
    total = len(in_list)-1
    if rand_limit > len(in_list):
        rand_limit = len(in_list)
    slots = []
    out = []
    while len(slots) <= rand_limit-1:
        r_num = random.randint(0, total)
        if r_num not in slots:
            slots.append(r_num)
    for slot_i in slots:
        out.append(in_list[slot_i])
    return out



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Object representation of a medialibrary file repository.

The object is a representation of all files under a directory tree.
The first method call is always to scan() to identify all directories
and files, which persist int he object memory.

It contains an object that does the actual scanning work
(which can be swapped out), and an object to do the loading
and creation of the media objects.
'''

from media.fileops.scanner import Walker


class Repo():
    '''
    The repository object.
    '''
    def __init__(self, in_path):
        self.root_path = in_path
        self.walker = Walker([in_path])
        self.dirs = []
        self.files = []

    def set_walker(self, in_object):
        '''
        For the rare occasion when someone wants to swap out the default
        walker object.
        '''
        self.walker = in_object

    def scan(self):
        '''
        Have the walker object scan the repo path.
        '''
        self.walker.scan()
        self.dirs = self.walker.dirs
        self.files = self.walker.files

    def file_match(self, in_pattern):
        '''
        Return all filenames matching a specific pattern.
        '''
        out = []
        for fname in self.files:
            if in_pattern.search(fname):
                out.append(fname)
        return out



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Main parser class for XML files'''

import xml.etree.ElementTree as ET
import xml.etree.ElementInclude as EI
import os.path
from os import chdir
import media.data.media
from media.xml.namespaces import Namespaces


class Parser():
    '''
    Main wrapper functions for all XML functionality against
    a given file.
    '''
    def __init__(self):
        self.medialist = []
        self.file_count = 0
        self.object_count = 0

    def load_file(self, in_filename):
        '''Load an XML file into the parser'''
        real_path = os.path.realpath(in_filename)
        dir_name = os.path.dirname(real_path)
        base_name = os.path.basename(real_path)
        chdir(dir_name)
        self.file_count += 1
        tree = ET.parse(base_name)
        root = tree.getroot()
        EI.include(root)
        xml_chunk = []
        for elem in root.findall('./media:media', Namespaces.ns):
            xml_chunk.append(media.data.media.Media(elem))
            self.object_count += 1
        return xml_chunk

    def stats(self):
        '''
        Report stats on how many files were read, and how many
        successful XML objects were created.
        '''
        return f"Files: {self.file_count} -- Objects {self.object_count}"



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Common code related to sorting operations.'''

# pylint: disable=too-few-public-methods

import string


def transform_string(in_value):
    '''Low level change to remove all punctuation from a string.'''
    if in_value is not None:
        no_punctuation = in_value.translate(
            in_value.maketrans("", "", string.punctuation))
        return no_punctuation.casefold()
    return "SHOULDNT BE HERE"


def build_filename_string(in_value):
    '''Convert all whitespace into underscores suitable for a filename'''
    level1 = transform_string(in_value)
    return level1.translate(level1.maketrans(" \t\n\r", "____"))


def build_sort_string(in_value):
    '''Build a string suitable for sorting, accounting for language rules.'''
    level1 = transform_string(in_value)
    word_split = level1.split()
    if word_split[0] in LanguageHelpers.Articles_English:
        article = word_split.pop(0)
        word_split.append('_'+article)
    return '_'.join(word_split)


class LanguageHelpers():
    '''Static data on languages.'''
    Articles_English = ['a', 'an', 'the']



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Objects for representation of proper nouns used in keywords.
'''

# pylint: disable=too-few-public-methods
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes

from media.xml.namespaces import Namespaces


class AbstractNoun():
    '''
    Root class for all nouns
    '''
    def __init__(self):
        self.value = ''
        self.sort_value = ''
        self.tagname = ''

    def __str__(self):
        return self.value

    def __hash__(self):
        return hash(self.value)

    def __lt__(self, other):
        return self.sort_value < other.sort_value

    def __rt__(self, other):
        return self.sort_value > other.sort_value

    def __eq__(self, other):
        return self.sort_value == other.sort_value


class Noun(AbstractNoun):
    '''
    Simplest class to represent proper nouns
    for Thing, Event, Group, Entity

    value represents the value that is displayed
    sort_value represents the value for sorting
    '''
    def __init__(self, in_element):
        super().__init__()
        self.value = in_element.text
        self.sort_value = self.value.casefold()
        self.tagname = Namespaces.ns_strip(in_element.tag)


class Place(AbstractNoun):
    '''
    ProperNoun class for a location

    Has attributes for every possible aspect
    of a location, which is probably going to be
    a problem.
    '''
    def __init__(self, in_place):
        super().__init__()
        self.generic = ''
        self.name = ''
        self.city = ''
        self.county = ''
        self.state = ''
        self.country = ''
        self.planet = ''
        if in_place is not None:
            self.tagname = Namespaces.ns_strip(in_place.tag)
            self._process(in_place)

    def _process(self, in_element):
        first_tag = True
        major = ''
        minor = ''
        for child in in_element:
            if first_tag:
                major = self._build_major_value(child)
                first_tag = False
            else:
                minor = self._build_minor_value(child, minor)
        if minor:
            minor = '(' + minor + ')'
            self.value = major + ' ' + minor
        else:
            self.value = major
        self.sort_value = self.value.casefold()

    def _build_major_value(self, in_element):
        tagname = Namespaces.ns_strip(in_element.tag)
        if tagname == 'generic':
            self.generic = in_element.text
        if tagname == 'name':
            self.name = in_element.text
        elif tagname == 'ci':
            self.city = in_element.text
        elif tagname == 'co':
            self.county = in_element.text
        elif tagname in ['st', 'pr']:
            self.state = in_element.text
        elif tagname == 'cn':
            self.country = in_element.text
        elif tagname == 'planet':
            self.planet = in_element.text
        major = in_element.text
        return major

    def _build_minor_value(self, in_element, minor):
        tagname = Namespaces.ns_strip(in_element.tag)
        if tagname == 'ci':
            self.city = in_element.text
        elif tagname == 'co':
            self.county = in_element.text
        elif tagname in ['st', 'pr']:
            self.state = in_element.text
        elif tagname == 'cn':
            self.country = in_element.text
        elif tagname == 'planet':
            self.planet = in_element.text
        if minor:
            minor += ', ' + in_element.text
        else:
            minor = in_element.text
        return minor


class Name(AbstractNoun):
    '''
    Proper noun for the name of a real person.

    A real person's name will include
    the common components like a given name,
    a family name, and maybe a middle name.

    This class is more heavily used since it
    is the standard name class for crew members
    or any other data types that use a name.
    '''
    def __init__(self, in_element):
        super().__init__()
        self.given = ''
        self.family = ''
        self.middle = ''
        self.sort = ''
        self.post_title = ''
        self.pre_title = ''
        if in_element is not None:
            self.tagname = Namespaces.ns_strip(in_element.tag)
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'gn':
                self.given = child.text
            elif tagname == 'fn':
                self.family = child.text
            elif tagname == 'mn':
                self.middle = child.text
            elif tagname == 'postTitle':
                self.post_title = child.text
            elif tagname == 'preTitle':
                self.pre_title = child.text
        self._build_value()
        self._build_sort_value()

    def _build_value(self):
        """Construct a printable name."""
        raw = ''
        if self.pre_title:
            raw += self.pre_title + ' '
        if self.given:
            raw += self.given + ' '
        if self.middle:
            raw += self.middle + ' '
        if self.family:
            raw += self.family
        if self.post_title:
            raw += ' ' + self.post_title
        self.value = raw

    def _build_sort_value(self):
        """Construct a string for sorting names."""
        raw = ''
        if self.family:
            raw = self.family.casefold() \
                    + '_' \
                    + self.given.casefold()
            if self.middle:
                raw += '_' + self.middle.casefold()
        else:
            raw = self.given.casefold()
            if self.middle:
                raw += '_' + self.middle
        if self.post_title:
            raw += '_' + self.post_title
        self.sort_value = raw

    def __str__(self):
        """The formal string value should be returned."""
        return self.value


class Art(AbstractNoun):
    """
    Proper noun describing a work of art.

    Element contains no child elements, only
    text, plus two optional attributes.
    """
    def __init__(self, in_art_element):
        super().__init__()
        self.art_type = ''
        self.art_year = 0
        if in_art_element is not None:
            self._process(in_art_element)

    def _process(self, in_element):
        art_name = in_element.text.strip()
        self.tagname = Namespaces.ns_strip(in_element.tag)
        if 'type' in in_element.attrib:
            self.art_type = in_element.attrib['type']
        if 'year' in in_element.attrib:
            self.art_year = int(in_element.attrib['year'])
        if self.art_type:
            self.value = art_name + ' (' + self.art_type + ')'
        else:
            self.value = art_name
        self.sort_value = self.value.casefold()



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
List out all movies, one per line.
'''

# pylint: disable=R0801

import os
import argparse
from media.tools.common import load_media_dev, compile_movies


class DebugList():
    '''Formatting a listing of movies, one per line.'''
    def __init__(self, in_movie):
        self.movie = in_movie
        self.output = ""
        self.output += self.mentry()

    @classmethod
    def list_header(cls):
        '''Generate a simple header'''
        out = f"{'Title':35s} {'Year':4s} {'Sort Title':35s} " + \
              f"{'Hash':20s} {'Object Address':18s}\n" + \
              f"{'=' * 35} {'=' * 4} {'=' * 20} {'=' * 18}"
        return out

    def mentry(self):
        '''One line entry'''
        y_string = ""
        if self.movie.catalog is not None:
            if self.movie.catalog.copyright is not None:
                y_string = self.movie.catalog.copyright.year
        hash_string = hash(self.movie)
        out = f"{self.movie.title!s:35s} {y_string:4d} " + \
              f"{self.movie.title.sort_title:35s} " + \
              f"{hash_string:20d} {hex(id(self.movie))}"
        return out

    def __str__(self):
        return self.output


def list_movies(movies):
    '''Provide a summary list of all movies'''
    table = {}
    print(DebugList.list_header())
    for movie in sorted(movies):
        movie_list_entry = DebugList(movie)
        print(movie_list_entry)
        if movie not in table:
            table[movie] = True
    print(f"Unique movie count: {len(table.keys())}")


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    devices = load_media_dev(mediapath)
    all_movies = compile_movies(devices)
    list_movies(all_movies)



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Code relating to the loading of XML files'''

# pylint: disable=too-few-public-methods

import media.xml.parser
from media.fileops.filenames import FilenameMatches


class Loader():
    '''Loads a predetermined list of media data files'''
    def __init__(self):
        self.file_count = 0
        self.object_count = 0
        self.parser = media.xml.parser.Parser()

    def load_media(self, repo, pattern=None):
        '''For each file, run the parser against it, and create an
           object if possible'''
        m_obj = []
        files = []
        if pattern is not None:
            files = repo.file_match(pattern)
        else:
            files = repo.file_match(FilenameMatches.All_Xml)
        for in_file in files:
            self.file_count += 1
            m_obj.extend(self.parser.load_file(in_file))
        self.object_count += len(m_obj)
        return m_obj



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Classes related to crewmembers of a piece
of visual art, like a movie or television show.
'''

# pylint: disable=too-few-public-methods
# pylint: disable=R0801

from media.xml.functions import xs_bool
from media.xml.namespaces import Namespaces
from media.data.nouns import Name


class Crew():
    '''
    Main container element for all crew objects.
    '''
    def __init__(self, in_element):
        self.directors = []
        self.editors = []
        self.writers = []
        self.cinemap = []
        self.cast = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'directors':
                self.directors = Directors.load(child)
            if tagname == 'editors':
                self.editors = SimpleCrew.load(child, 'editor')
            if tagname == 'cinemaphotographers':
                self.cinemap = SimpleCrew.load(child, 'cinemaphotographer')
            if tagname == 'writers':
                self.writers = SimpleCrew.load(child, 'writer')
            if tagname == 'cast':
                self.cast = Cast(child)


class Directors():
    '''
    Simple class method to parse director elements.
    '''
    @classmethod
    def load(cls, in_element):
        ''' Load an array of names for a given element in the XML'''
        out = []
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'director':
                    out.append(Name(child))
        return out


class Writers():
    '''
    Class method to parser writer elements, which
    are a little more complex compared to other
    crew elements.
    '''
    @classmethod
    def load(cls, in_element):
        ''' Load an array of writers based on the XML'''
        out = []
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'writer':
                    out.append(Name(child))


class SimpleCrew():
    '''
    Generic crew loader object, which returns
    a list of Name elements.
    '''
    @classmethod
    def load(cls, in_element, desired_tag):
        '''Load an array of names for a given element in the XML'''
        out = []
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == desired_tag:
                    out.append(Name(child))
        return out

    @classmethod
    def output(cls, title, in_list):
        '''Special output formatter to string multiple names together'''
        out_string = ""
        if len(in_list) == 1:
            out_string = f"{title}: {in_list[0]}"
        else:
            out_string = f"{title}"
            for name in in_list:
                out_string += f"{name}. "
            out_string = out_string[:-2]
        return out_string


class Cast():
    '''
    Container object for all cast members, or a subset of cast members.
    '''
    def __init__(self, in_element):
        self.cast = []
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'role':
                    self.cast.append(Role(child))


class Role():
    '''
    A role contains a single actor person, and one or more
    characters portrated by the role.
    '''
    def __init__(self, in_element):
        self.actor = None
        self.portrays = []
        self.billing = 1
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'actor':
                    self.actor = Name(child)
                if tagname == 'as':
                    child_tag = Namespaces.ns_strip(child[0].tag)
                    if child_tag == 'narrator':
                        self.portrays.append(PortraysNarrator())
                    elif child_tag == 'self':
                        self.portrays.append(PortraysSelf(child))
                    elif child_tag == 'title':
                        self.portrays.append(CharacterTitle(child))
                    else:
                        self.portrays.append(CharacterName(child))
            if 'billing' in in_element.attrib:
                self.billing = int(in_element.attrib['billing'])

    def __str__(self):
        return "Role monster"


class Portrays():
    '''
    Abstract root class for all portrays objects.

    '''
    def __init__(self):
        self.value = ''
        self.formal = ''
        self.sort_value = ''

    def __str__(self):
        return self.value

    def __hash__(self):
        '''
        Hash values are only guaraunteed to be unique
        against other values against the same actor within
        the same role structure.
        '''
        return hash(self.formal)

    def __lt__(self, other):
        return self.sort_value < other.sort_value

    def __rt__(self, other):
        return self.sort_value > other.sort_value

    def __eq__(self, other):
        return self.sort_value == other.sort_value


class PortraysNarrator(Portrays):
    '''
    Class designating the special role Narrator
    '''
    def __init__(self):
        super().__init__()
        self.value = 'Narrator'
        self.formal = 'Narrator'
        self.sort_value = 'narrator'


class PortraysSelf(Portrays):
    '''
    Class designed for the special role Self
    '''
    def __init__(self, in_element):
        super().__init__()
        self.value = 'Self'
        self.formal = 'Self'
        self.sort_value = 'self'
        ref_tag = in_element[0]
        if 'fictionalVariant' in ref_tag.attrib:
            if xs_bool(ref_tag.attrib['fictionalVariant']):
                self.formal += ' (fictional variant)'
        if 'archivalFootage' in ref_tag.attrib:
            if xs_bool(ref_tag.attrib['archivalFootage']):
                self.formal += ' (archival footage)'


class CharacterTitle(Portrays):
    '''
    Class designed for the special role Title
    '''
    def __init__(self, in_element):
        super().__init__()
        self.value = in_element[0].text
        self.formal = in_element[0].text
        self.sort_value = in_element[0].text.casefold()


class CharacterName(Portrays):
    '''
    Name class for a character name in a movie

    This class is loosely similar to a Name class, but
    there are variations where the character in a
    movie may have names that don't conform to
    the naming conventions of a real person.
    '''
    name_matrix = {
            'gn': 'given',
            'nick': 'nick',
            'mn': 'middle',
            'fn': 'family'
    }

    def __init__(self, in_element):
        super().__init__()
        self.value = ''
        self.chunk = {}
        self.pre_title = ''
        self.post_title = ''
        self.variant = ''
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        '''Build the object based on the data'''
        order = []
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            tagcount = 1
            if tagname in CharacterName.name_matrix:
                if tagname == 'nick':
                    self.chunk[CharacterName.name_matrix[tagname]] = "'" + \
                            child.text + "'"
                else:
                    self.chunk[CharacterName.name_matrix[tagname]] = child.text
                order.append(CharacterName.name_matrix[tagname])
                tagcount += 1
            elif tagname == 'preTitle':
                self.pre_title = child.text
            elif tagname == 'postTitle':
                self.post_title = child.text
            elif tagname == 'variant':
                self.variant = child.text
        self._build_value(order)
        self._build_formal()
        self._build_sort(order)

#    def _process2(self, in_element):
#        '''Build the object based on the data'''
#        for child in in_element:
#            tagname = Namespaces.ns_strip(child.tag)
#            tagcount = 1
#            order = []
#            if tagname == 'self':
#                self.chunk['self'] = "(self)"
#                order = ['self']
#                self._build_value(order)
#                return
#            if tagname == 'narrator':
#                self.chunk['narrator'] = "(narrator)"
#                order = ['narrator']
#                self._build_value(order)
#                return
#            if tagname == 'title':
#                self.chunk['title'] = child.text
#                order = ['title']
#                self._build_value(order)
#                return
#            if tagname == 'nick':
#                self.chunk['nick'] = child.text
#                order.append('nick')
#            if tagname == 'gn':
#                self.chunk['given'] = child.text
#                order.append('given')
#            if tagname == 'mn':
#                self.chunk['middle'] = child.text
#                order.append('middle')
#            if tagname == 'fn':
#                self.chunk['family'] = child.text
#                order.append('family')
#            tagcount += 1
#        self._build_value(order)
#        return

    def _build_value(self, order):
        raw = ''
        for chunk_i in order:
            raw += self.chunk[chunk_i] + ' '
        raw = raw[:-1]
        self.value = raw

    def _build_formal(self):
        raw = ''
        if self.pre_title:
            raw = self.pre_title + ' ' + self.value
        else:
            raw = self.value
        if self.post_title:
            raw += ' ' + self.post_title
        if self.variant:
            raw += ' (' + self.variant + ')'
        self.formal = raw

    def _build_sort(self, order):
        order = ['family', 'given', 'middle']
        raw = ''
        for o_i in order:
            if o_i in self.chunk:
                raw += self.chunk[o_i].casefold() + '_'
        raw = raw[:-1]
        self.sort_value = raw

#    def complete(self):
#        out = ''
#        if self.pre_title:
#            out = self.pre_title + ' ' + self.value
#        else:
#            out = self.value
#        if self.post_title:
#            out += ' ' + self.post_title
#        if self.variant:
#            out += ' (' + self.variant + ')'
#        return out

    def __str__(self):
        return f"{self.value}"

    def __repr__(self):
        return f"{self.chunk['given']} {self.chunk['family']}"



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''XML Namespace Constants'''


class Namespaces():
    '''Static data on XML Namespaces'''
    ns = {
            "media": "http://vectortron.com/xml/media/media",
            "movie": "http://vectortron.com/xml/media/movie"
        }

    @classmethod
    def nsf(cls, in_tag):
        '''return the fully qualified namespace to the prefix format'''
        return '{' + Namespaces.ns[in_tag] + '}'

    @classmethod
    def ns_strip(cls, in_tag):
        '''Strip the namespace from an element'''
        out = in_tag.split("}", 1)
        return out[1]



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
List out all movie genres, grouped by primary and secondary genres.
'''

# pylint: disable=R0801

import os
import argparse
import random
from media.tools.common import load_media_dev, compile_movies


class PrimaryBucket():
    '''
    Bucket class system for primary genres.

    Very simple dictionary where key values are the primary genre name, and
    the value is a list containing the movie objects.
    '''
    def __init__(self):
        self.genres = {}
        self.movie_count = 0

    def add(self, genre, movie):
        '''
        Add a movie to a genre, regardless of whether it exists or not.
        '''
        if genre in self.genres:
            self.genres[genre].append(movie)
        else:
            self.genres[genre] = [movie]
        self.movie_count += 1

    def bucket(self, genre):
        '''
        Return the list for a specific genre.
        '''
        if genre in self.genres:
            return self.genres[genre]
        return None

    def report(self):
        '''
        Generate a primary genre report.
        '''
        out = f"{' ' * 55} {'<<< Primary Genres >>>'}\n\n"
        out += f"{'Primary Genre':15s} {'Count':5s} {'Perc':6s} " + \
               f"{'Ratio':50s} {'Sample Title'}\n"
        out += f"{'-' * 15}  {'-' * 5} {'-' * 6} {'-' * 50} {'-' *45}\n"
        for genre in sorted(self.genres):
            pg_tally = len(self.genres[genre])
            perc = float(pg_tally / self.movie_count * 100)
            text_pro_bar = proportion_bar(pg_tally, self.movie_count)
            rand_movie = random_movie(self.bucket(genre))
            out += f"{genre:15s}  {pg_tally:5d} {perc:5.1f}% " + \
                   f"{text_pro_bar:50s} {rand_movie.title!s}\n"
        out += f"{'-' * 15}  {'-' * 5} {'-' * 6} {'-' * 50} {'-' *45}\n"
        out += f"Total movie count {self.movie_count}\n\n"
        return out


class SecondaryBucket():
    '''
    A class for a primary/secondary bucket tree.

    A slightly more complex dictionary compared to PrimaryBucket.
    Each key is still based on the value of 'primary', but the value is another
    dictionary, with each key of the smaller dictionary being the values of
    the 'secondary' elements, and the value being a list of movie objects.
    '''
    def __init__(self):
        self.genres = {}
        self.movie_count = 0

    def add(self, in_primary, in_secondaries, movie):
        '''
        Populate the primary/secondary buckets.
        '''
        if len(in_secondaries) == 0:
            self._add_no_secondaries(in_primary, movie)
        else:
            self._add_with_secondaries(in_primary, in_secondaries, movie)
        self.movie_count += 1

    def _add_no_secondaries(self, in_primary, movie):
        '''
        Add a movie to a special secondary bucket signifying
        that there are no secondary genres for this movie.
        '''
        sec_genre = '(none)'
        if in_primary in self.genres:
            if sec_genre in self.genres[in_primary]:
                self.genres[in_primary][sec_genre].append(movie)
            else:
                self.genres[in_primary][sec_genre] = [movie]
        else:
            self.genres[in_primary] = {sec_genre: [movie]}

    def _add_with_secondaries(self, in_primary, in_secondaries, movie):
        '''
        Add a movie to every possible secondary genre bucket
        for a given primary genre bucket.
        '''
        for sec_genre in in_secondaries:
            if in_primary in self.genres:
                if sec_genre in self.genres[in_primary]:
                    self.genres[in_primary][sec_genre].append(movie)
                else:
                    self.genres[in_primary][sec_genre] = [movie]
            else:
                self.genres[in_primary] = {sec_genre: [movie]}

    def _count_primary_unique(self, in_primary):
        '''
        We need this to count movies under a primary genre
        that may fall into multiple secondary buckets.

        This function creates a list and iterates through
        all of the movies, but only adds the movie object to
        the list if it isn't already present.
        '''
        out = []
        for secondary_i in self.genres[in_primary]:
            for movie in self.genres[in_primary][secondary_i]:
                if movie not in out:
                    out.append(movie)
        return len(out)

    def _count_secondary_occurences(self, in_primary_g):
        sec_occur = 0
        for prim_g in self.genres:
            for sec_g in self.genres[prim_g]:
                if sec_g == in_primary_g:
                    # sec_occur += 1
                    sec_occur += len(self.genres[prim_g][sec_g])
        return sec_occur

    def bucket(self, in_primary, in_secondary):
        '''
        Quick routine to get to a specific bucket.
        '''
        if in_primary in self.genres:
            if in_secondary in self.genres[in_primary]:
                return self.genres[in_primary][in_secondary]
        return None

    def report(self):
        '''
        Generate a secondary genre report.
        '''
        out = f"{' ' * 54} {'<<< Secondary Genres >>>'}\n\n"
        for primary_i in sorted(self.genres):
            sec_tally = 0
            for secondary_i in self.genres[primary_i]:
                sec_tally += len(self.genres[primary_i][secondary_i])
            prim_count = self._count_primary_unique(primary_i)
            sec_occur = self._count_secondary_occurences(primary_i)
            out += f"\nPrimary Genre: {primary_i:12s}  ({prim_count:d} / " + \
                   f"{self.movie_count:d}) "
            out += f"Occurences as a secondary genre: {sec_occur}\n\n"
            out += f"{'Secondary Genre':12s} {'Count':5s} {'Perc':6s} " + \
                   f"{'Ratio':50s} {'Sample Title'}\n"
            out += f"{'-' * 15} {'-' * 5} {'-' * 6} {'-' * 50} {'-' * 45}\n"
            for secondary_i in sorted(self.genres[primary_i]):
                sec_count = len(self.genres[primary_i][secondary_i])
                perc = float(sec_count / sec_tally * 100)
                text_pro_bar = proportion_bar(sec_count, sec_tally)
                rand_movie = random_movie(self.genres[primary_i][secondary_i])
                out += f"{secondary_i:15s} {sec_count:5d} {perc:5.1f}% " + \
                       f"{text_pro_bar:50s} {rand_movie.title!s}\n"
            out += f"{'-' * 15} {'-' * 5} {'-' * 6} {'-' * 50} {'-' * 45}\n"
        return out


def report_header():
    '''
    Print a simplified report header.
    '''
    out = f"{'=' * 132}\n\n"
    out += f" {'Movie Genre Report'}\n\n"
    out += f"{'=' * 132}\n\n"
    return out


def populate_primary_buckets(movies):
    '''Put the primary genre of every movie in a bucket.'''
    pg_bucket = PrimaryBucket()
    for movie in movies:
        if movie.classification.genres:
            prim_genre = movie.classification.genres.primary
            pg_bucket.add(prim_genre, movie)
    return pg_bucket


def populate_secondary_buckets(movies):
    '''Put the primary and secondary genres of every movie in a bucket.'''
    sg_bucket = SecondaryBucket()
    for movie in movies:
        if movie.classification.genres:
            prim_genre = movie.classification.genres.primary
            sec_genre = movie.classification.genres.secondary
            sg_bucket.add(prim_genre, sec_genre, movie)
    return sg_bucket


def proportion_bar(value, maximum, limit=50):
    '''
    Generate a proportional text bar based on a set of values, including
    the desired value, the maximum possible value, and the
    character limit on how long the bar can be.
    '''
    ratio = int(value / maximum * limit)
    tics = ratio * '-'
    return f"{tics}"


def random_movie(in_list):
    '''
    For a given list of movies, return one random
    movie from the list.
    '''
    total_count = len(in_list)
    if total_count == 1:
        random_index = 0
    else:
        random_index = random.randint(0, len(in_list)-1)
    return in_list[random_index]


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    devices = load_media_dev(mediapath)
    all_movies = compile_movies(devices)
    primary = populate_primary_buckets(all_movies)
    secondary = populate_secondary_buckets(all_movies)
    print(report_header())
    print(primary.report())
    print(secondary.report())
